/*
Dozen Days of Dice episode 11: Everest
Copyright (c) 2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.everest
{
	import flash.display.SimpleButton;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import com.blazinggames.dozendays.*;
	import com.blazinggames.dozendays.dice.*;

	public class EverestGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const ROLL_TEXT:String = "Roll!";
		public static const DONE_TEXT:String = "Done!";
		public static const SCORE_FORMAT:TextFormat = new TextFormat(null, 18, 0x0FFFFFF, null, null, null, null, null, TextFormatAlign.CENTER);
		public static const PLAYER_FORMAT:TextFormat = new TextFormat(null, 18, 0x0FFFFFF, null, null, null, null, null, TextFormatAlign.CENTER);

		private static const DIE_Y_UNSELECTED:int = 380;
		private static const DIE_Y_SELECTED:int = 340;
		
// ***** VARIABLES **********************************************************

		protected var _manager:IGameMode;
		protected var _setup:PlayerSetup;
		protected var _rollButton:TextButton;
		protected var _doneButton:TextButton;
		protected var _toTitle:TextButton;
		protected var _restartGame:TextButton;
		
		protected var _flags:Array;
		protected var _dice:Array;
		protected var _selected:Array;
		protected var _scoreButtons:Array;
		protected var _player:int;
		protected var _winFlag:Flag;
		protected var _playerText:TextField;

// ***** CONSTRUCTOR ********************************************************

		public function EverestGame(manager:IGameMode, setup:PlayerSetup)
		{
			var cntr:int, cntrPlayer:int;
			_manager = manager;
			_setup = setup;
			render();

			_rollButton = new TextButton(ROLL_TEXT, 170, 40);
			_rollButton.x = 210;
			_rollButton.y = 200;
			addChild(_rollButton);
			_rollButton.addEventListener(MouseEvent.MOUSE_UP, handleRoll);
			
			_doneButton = new TextButton(DONE_TEXT, 90, 40);
			_doneButton.x = 250;
			_doneButton.y = 250;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleDone);

			_winFlag = new Flag();
			_winFlag.x = 288;
			_winFlag.y = 0;
			_winFlag.width = 64;
			_winFlag.height = 64;
			addChild(_winFlag);
			
			_playerText = new TextField();
			_playerText.x = _playerText.y = 0;
			_playerText.width = 640;
			_playerText.height = 50;
			_playerText.defaultTextFormat = PLAYER_FORMAT;
			addChild(_playerText);
			
			_dice = new Array(3);
			_selected = new Array(3);
			for (cntr = 0; cntr < 3; ++cntr) {
				_dice[cntr] = new GenericDie();
				_dice[cntr].x = 210 + 60 * cntr;
				_dice[cntr].y = DIE_Y_UNSELECTED;
				_dice[cntr].width = 50;
				_dice[cntr].height = 50;
				_dice[cntr].addEventListener(MouseEvent.MOUSE_UP, handleSelections);
				addChild(_dice[cntr]);
				_selected[cntr] = false;
			}
			
			_flags = new Array(24);
			_scoreButtons = new Array(24);
			var tempFlag:Flag;
			var baseXLeft:int;
			var baseXRight:int;
			var baseY:int;
			for (cntr = 0; cntr < 12; ++cntr) {
				baseXLeft = 0 + 8 * cntr;
				baseXRight = 416 - 8 * cntr;
				baseY = 400 - 32 * cntr; 
				_flags[cntr] = new Array(6);
				_flags[23-cntr] = new Array(6);
				for (cntrPlayer = 0; cntrPlayer < 6; ++cntrPlayer) {
					tempFlag = new Flag(PlayerSetup.PLAYER_COLORS[cntrPlayer],PlayerSetup.PLAYER_BACKGROUNDS[cntrPlayer]);
					tempFlag.x = baseXLeft + 32 * cntrPlayer;
					tempFlag.y = baseY;
					addChild(tempFlag);
					_flags[cntr][cntrPlayer] = tempFlag;
					tempFlag = new Flag(PlayerSetup.PLAYER_COLORS[cntrPlayer],PlayerSetup.PLAYER_BACKGROUNDS[cntrPlayer]);
					tempFlag.x = baseXRight + 32 * cntrPlayer;
					tempFlag.y = baseY;
					addChild(tempFlag);
					_flags[23 - cntr][cntrPlayer] = tempFlag;
				}
				_scoreButtons[cntr] = new TextButton(""+(cntr+1), 32, 32);
				_scoreButtons[cntr].x = baseXLeft +172;
				_scoreButtons[cntr].y = baseY;
				_scoreButtons[cntr].addEventListener(MouseEvent.MOUSE_UP, handleSetScore);
				_scoreButtons[23 - cntr] = new TextButton(""+(cntr+1), 32, 32);
				_scoreButtons[23 - cntr].x = baseXRight - 12;
				_scoreButtons[23 - cntr].y = baseY;
				_scoreButtons[23 - cntr].addEventListener(MouseEvent.MOUSE_UP, handleSetScore);
				addChild(_scoreButtons[cntr]);
				addChild(_scoreButtons[23 - cntr]);
			}

			_toTitle = new TextButton("return to title...", 310, 20, SCORE_FORMAT);
			_toTitle.y = 460;
			addChild(_toTitle);
			_toTitle.addEventListener(MouseEvent.MOUSE_UP, handleToTitle);

			_restartGame = new TextButton("Start a new Round", 310, 20, SCORE_FORMAT);
			_restartGame.x = 330;
			_restartGame.y = 460;
			addChild(_restartGame);
			_restartGame.addEventListener(MouseEvent.MOUSE_UP, handleRestart);
			_restartGame.visible = false;
			startGame();
		}

// ***** Functions - Event Handling *****************************************

		private function handleSelections(evt:Event):void
		{
			var die:int = -1;
			var score:int = 0;
			var cntr:int;
			
			for (cntr = 0; cntr < _dice.length; ++cntr) {
				if (_dice[cntr] == evt.target)
					die = cntr;
			}
			if (die < 0) 
				return;
			_selected[die] = ! _selected[die];
			for (cntr = 0; cntr < _dice.length; ++cntr) {
				if (_selected[cntr]) {
					_dice[cntr].y = DIE_Y_SELECTED;
					score += _dice[cntr].getValue();
				} else
					_dice[cntr].y = DIE_Y_UNSELECTED;
			}

			for (cntr = 0; cntr < 12; ++cntr) {
				if (score == (cntr+1)) {
					_scoreButtons[cntr].visible = !_flags[cntr][_player].visible;
					_scoreButtons[23-cntr].visible = !_flags[23-cntr][_player].visible;
				} else {
					_scoreButtons[cntr].visible = false;
					_scoreButtons[23 - cntr].visible = false;
				}
			}
			
		}
		
		private function handleRoll(evt:Event):void
		{
			var cntr:int;
			
			for (cntr = 0; cntr < _dice.length; ++cntr) {
				_dice[cntr].quickRoll();
				_dice[cntr].y = DIE_Y_UNSELECTED;
				_dice[cntr].visible = true;
			}
			_rollButton.visible = false;
			_doneButton.visible = true;
		}

		private function handleDone(evt:Event):void
		{
			nextPlayer();
		}

		public function handleSetScore(evt:MouseEvent):void
		{
			var cntr:int, scoreChoice:int = -1;
			for (cntr = 0; cntr < _scoreButtons.length; ++cntr) {
				if (evt.target == _scoreButtons[cntr])
					scoreChoice = cntr;
			}
			if (scoreChoice < 0)
				return;
			_flags[scoreChoice][_player].visible = true;
			for (cntr = 0; cntr < 3; ++cntr) {
				if (_selected[cntr] == true) {
					_selected[cntr] = false;
					_dice[cntr].visible = false;
				}
			}
			for (cntr = 0; cntr < _scoreButtons.length; ++cntr)
				_scoreButtons[cntr].visible = false;
		}
		
		public function handleToTitle(evt:MouseEvent):void
		{
			_manager.changeMode(1);
		}

		public function handleRestart(evt:MouseEvent):void
		{
			startGame();
		}

		
// ***** Functions - Rendering **********************************************

		private function render():void
		{
			graphics.clear();
			graphics.beginFill(0xA0A0FF);
            graphics.drawRect(0, 0, 640, 480);
            graphics.endFill();

			graphics.beginFill(0x808080);
			graphics.moveTo(170,480);
			graphics.lineTo(300,0);
			graphics.lineTo(430, 480);
			graphics.lineTo(170,480);
			graphics.endFill();

		}

// ***** Functions - Gameplay management ************************************

		public function startGame():void
		{
			var cntr:int, cntrPlayer:int;

			// first make sure not a null game
			var validGame:Boolean = false;
			
			for (cntr = 0; cntr < 6; ++cntr)
				if (_setup.getPlayerType(cntr) == PlayerSetup.PLAYER_HUMAN)
					validGame = true;
			
			if ( !validGame) {
				_rollButton.visible = false;
				_doneButton.visible = false;
				return;
			}
			
			_restartGame.visible = false;
			_winFlag.visible = false;
			for (cntr = 0; cntr < 12; ++cntr) {
				for (cntrPlayer = 0; cntrPlayer < 6; ++cntrPlayer) {
					_flags[cntr][cntrPlayer].visible = false;
					_flags[23 - cntr][cntrPlayer].visible = false;
				}
				_scoreButtons[cntr].visible = false;
				_scoreButtons[23 - cntr].visible = false;
			}
			_player = -1;
			nextPlayer();
		}
		
		public function nextPlayer():void
		{
			var cntr:int;

			if (_player >= 0) {
				var winCheck:Boolean = true;
				for (cntr = 0; cntr < _flags.length; ++cntr)
					if (_flags[cntr][_player].visible == false)
						winCheck = false;
				if (winCheck) {
					_winFlag.setColors(PlayerSetup.PLAYER_COLORS[_player],PlayerSetup.PLAYER_BACKGROUNDS[_player]);
					_winFlag.visible = true;
					_restartGame.visible = true;
					return;
				}
			}
			_rollButton.visible = true;
			_doneButton.visible = false;
			for (cntr = 0; cntr < _dice.length; ++cntr)
				_dice[cntr].visible = _selected[cntr] = false;
			for (cntr = 0; cntr < _scoreButtons.length; ++cntr)
				_scoreButtons[cntr].visible = false;
			++_player;
			if (_player > 5) {
				_player = 0;
			}
			if (_setup.getPlayerType(_player) == PlayerSetup.PLAYER_COMPUTER){
				AI();
				nextPlayer();
			} else if (_setup.getPlayerType(_player) == PlayerSetup.PLAYER_NONE)
				nextPlayer();
			_playerText.textColor = PlayerSetup.PLAYER_COLORS[_player];
			_playerText.text = _setup.getPlayerName(_player);
		}
		
		private function AIScore(n:int):Boolean
		{
			if ((n < 1) || (n > 12))
				return false;
			if (_flags[n-1][_player].visible == false) {
				_flags[n-1][_player].visible = true;
				return true;
			} else if (_flags[24 - n][_player].visible == false) {
				_flags[24 - n][_player].visible = true;
				return true;
			} else
				return false;
		}

		private function AI():void
		{
			var cntr:int, score:int;
			
			for (cntr = 0; cntr < _dice.length; ++cntr)
				_dice[cntr].quickRoll();
				
			if (AIScore(_dice[0].getValue() + _dice[1].getValue() + _dice[2].getValue()))
				return;
			else if (AIScore(_dice[0].getValue() + _dice[1].getValue()))
				AIScore(_dice[2].getValue());
			else if (AIScore(_dice[0].getValue() + _dice[2].getValue()))
				AIScore(_dice[1].getValue());
			else if (AIScore(_dice[1].getValue() + _dice[2].getValue()))
				AIScore(_dice[0].getValue());
			else {
				AIScore(_dice[0].getValue());
				AIScore(_dice[1].getValue());
				AIScore(_dice[2].getValue());
			}
		}
	} // class
} // package
